﻿using System;
using System.IO;

using Popolo.ThermophysicalProperty;
using Popolo.ThermalLoad;
using Popolo.Weather;

namespace TestProject
{
  class Program
  {
    static void Main(string[] args)
    {
      simulateHeatLoad2();
    }

    static void moistAirTest()
    {
      while (true)
      {
        Console.WriteLine("Enter the dry-bulb temperature.");
        string tmpS = Console.ReadLine();
        double tmp = double.Parse(tmpS);

        double hrt = 1000 * MoistAir.GetSaturationHumidityRatioFromDryBulbTemperature(tmp, 101.325);
        double ent = MoistAir.GetSaturationEnthalpyFromDrybulbTemperature(tmp, 101.325);

        Console.WriteLine("Saturation humidity ratio = " + hrt.ToString("F1") + "g/kg");
        Console.WriteLine("Saturation enthalpy = " + ent.ToString("F1") + "kJ/kg");
        Console.WriteLine();
      }
    }

    static void simulateFreeFloat()
    {
      //建物モデルを作成
      BuildingThermalModel bModel = makeBuildingModel();

      //1年間の気象データを作成
      RandomWeather weather = new RandomWeather(0, RandomWeather.Location.Tokyo);
      weather.MakeWeather(1, out double[] dbt, out double[] hmd, out double[] rad, out bool[] isFair);

      //日時と太陽を作成
      DateTime dTime = new DateTime(2019, 1, 1, 0, 0, 0);
      Sun sun = new Sun(Sun.City.Tokyo);
      ImmutableZone zone = bModel.MultiRoom[0].Zones[0]; //ゾーンを取得

      //計算実行
      using (StreamWriter sWriter = new StreamWriter("out.csv"))
      {
        //タイトル行
        sWriter.WriteLine("Date time, Drybulb temperature [C], Humidity ratio [g/kg]");

        //年間計算実行
        for (int hour = 0; hour < dbt.Length; hour++)
        {
          //太陽位置を更新して直散分離する
          sun.Update(dTime);
          sun.SeparateGlobalHorizontalRadiation(rad[hour], Sun.SeparationMethod.Erbs);

          //夜間放射[W/m2]を計算する
          double vp = MoistAir.GetWaterVaporPartialPressureFromHumidityRatio(0.001 * hmd[hour], 101.325);
          double noc = Sky.GetNocturnalRadiation(dbt[hour], isFair[hour] ? 0 : 10, vp);

          //外気条件を設定する
          bModel.UpdateOutdoorCondition(dTime, sun, dbt[hour], 0.001 * hmd[hour], noc);

          //モデルを更新する
          bModel.ForecastHeatTransfer();
          bModel.ForecastWaterTransfer();
          bModel.FixState();

          //ゾーンの温湿度を書き出す
          sWriter.WriteLine(
            dTime.ToString("MM/dd HH:mm") + "," +
            zone.Temperature.ToString("F2") + "," +
            (1000 * zone.HumidityRatio).ToString("F2"));

          //時刻を1時間進める
          dTime = dTime.AddHours(1);
        }
      }
    }

    static void simulateHeatLoad()
    {
      //建物モデルを作成
      BuildingThermalModel bModel = makeBuildingModel();

      //1年間の気象データを作成
      RandomWeather weather = new RandomWeather(0, RandomWeather.Location.Tokyo);
      weather.MakeWeather(1, out double[] dbt, out double[] hmd, out double[] rad, out bool[] isFair);

      //日時と太陽を作成
      DateTime dTime = new DateTime(2019, 1, 1, 0, 0, 0);
      Sun sun = new Sun(Sun.City.Tokyo);
      ImmutableZone zone = bModel.MultiRoom[0].Zones[0]; //ゾーンを取得

      //計算実行
      using (StreamWriter sWriter = new StreamWriter("out.csv"))
      {
        //タイトル行
        sWriter.WriteLine(
          "Date time, Drybulb temperature [C], Humidity ratio [g/kg]," +
          "Sensible heat load [kW], Latent heat load [kW]");

        //年間計算実行
        for (int hour = 0; hour < dbt.Length; hour++)
        {
          //太陽位置を更新して直散分離する
          sun.Update(dTime);
          sun.SeparateGlobalHorizontalRadiation(rad[hour], Sun.SeparationMethod.Erbs);

          //夜間放射[W/m2]を計算する
          double vp = MoistAir.GetWaterVaporPartialPressureFromHumidityRatio(0.001 * hmd[hour], 101.325);
          double noc = Sky.GetNocturnalRadiation(dbt[hour], isFair[hour] ? 0 : 10, vp);

          //外気条件を設定する
          bModel.UpdateOutdoorCondition(dTime, sun, dbt[hour], 0.001 * hmd[hour], noc);

          //空調条件を設定する
          bool weekDay = dTime.DayOfWeek != DayOfWeek.Saturday && dTime.DayOfWeek != DayOfWeek.Sunday;
          if (weekDay && 8 <= dTime.Hour && dTime.Hour < 20)
          {
            //空調時間帯は25度、0.010kg/kgに制御
            bModel.ControlDrybulbTemperature(0, 0, 25);
            bModel.ControlHumidityRatio(0, 0, 0.010);
          }
          else
          {
            //非空調時間帯は熱供給は0
            bModel.ControlHeatSupply(0, 0, 0);
            bModel.ControlWaterSupply(0, 0, 0);
          }

          //モデルを更新する
          bModel.ForecastHeatTransfer();
          bModel.ForecastWaterTransfer();
          bModel.FixState();

          //ゾーンの温湿度を書き出す
          sWriter.WriteLine(
            dTime.ToString("MM/dd HH:mm") + "," +
            zone.Temperature.ToString("F2") + "," +
            (1000 * zone.HumidityRatio).ToString("F2") + "," +
            (0.001 * zone.HeatSupply).ToString("F2") + "," + 
            (2500 * zone.WaterSupply).ToString("F2"));

          //時刻を1時間進める
          dTime = dTime.AddHours(1);
        }
      }
    }

    static void simulateHeatLoad2()
    {
      //建物モデルを作成
      BuildingThermalModel bModel = makeBuildingModel();

      //1年間の気象データを作成
      RandomWeather weather = new RandomWeather(0, RandomWeather.Location.Tokyo);
      weather.MakeWeather(1, out double[] dbt, out double[] hmd, out double[] rad, out bool[] isFair);

      //日時と太陽を作成
      DateTime dTime = new DateTime(2019, 1, 1, 0, 0, 0);
      Sun sun = new Sun(Sun.City.Tokyo);
      ImmutableZone zone = bModel.MultiRoom[0].Zones[0]; //ゾーンを取得

      //計算実行
      using (StreamWriter sWriter = new StreamWriter("out.csv"))
      {
        //タイトル行
        sWriter.WriteLine(
          "Date time, Drybulb temperature [C], Humidity ratio [g/kg]," +
          "Sensible heat load [kW], Latent heat load [kW]");

        //年間計算実行
        for (int hour = 0; hour < dbt.Length; hour++)
        {
          //太陽位置を更新して直散分離する
          sun.Update(dTime);
          sun.SeparateGlobalHorizontalRadiation(rad[hour], Sun.SeparationMethod.Erbs);

          //夜間放射[W/m2]を計算する
          double vp = MoistAir.GetWaterVaporPartialPressureFromHumidityRatio(0.001 * hmd[hour], 101.325);
          double noc = Sky.GetNocturnalRadiation(dbt[hour], isFair[hour] ? 0 : 10, vp);

          //外気条件を設定する
          bModel.UpdateOutdoorCondition(dTime, sun, dbt[hour], 0.001 * hmd[hour], noc);

          //冷暖房能力の制限を設定
          if (5 <= dTime.Month && dTime.Month <= 10)
          {
            bModel.SetCoolingCapacity(0, 0, 3000);
            bModel.SetDehumidifyingCapacity(0, 0, 1d / 2500d);
            bModel.SetHeatingCapacity(0, 0, 0);
            bModel.SetHumidifyingCapacity(0, 0, 0);
          }
          else 
          {
            bModel.SetCoolingCapacity(0, 0, 0);
            bModel.SetDehumidifyingCapacity(0, 0, 0);
            bModel.SetHeatingCapacity(0, 0, 3000);
            bModel.SetHumidifyingCapacity(0, 0, 1d / 2500d);
          }

          //空調条件を設定する
          bool weekDay = dTime.DayOfWeek != DayOfWeek.Saturday && dTime.DayOfWeek != DayOfWeek.Sunday;
          if (weekDay && 8 <= dTime.Hour && dTime.Hour < 20)
          {
            //空調時間帯は25度、0.010kg/kgに制御
            bModel.ControlDrybulbTemperature(0, 0, 25);
            bModel.ControlHumidityRatio(0, 0, 0.010);
          }
          else
          {
            //非空調時間帯は熱供給は0
            bModel.ControlHeatSupply(0, 0, 0);
            bModel.ControlWaterSupply(0, 0, 0);
          }

          //モデルを更新する
          bModel.UpdateHeatTransferWithinCapacityLimit();

          //ゾーンの温湿度を書き出す
          sWriter.WriteLine(
            dTime.ToString("MM/dd HH:mm") + "," +
            zone.Temperature.ToString("F2") + "," +
            (1000 * zone.HumidityRatio).ToString("F2") + "," +
            (0.001 * zone.HeatSupply).ToString("F2") + "," +
            (2500 * zone.WaterSupply).ToString("F2"));

          //時刻を1時間進める
          dTime = dTime.AddHours(1);
        }
      }
    }

    static BuildingThermalModel makeBuildingModel()
    {
      //傾斜面を作成
      Incline incS = new Incline(Incline.Orientation.S, 0.5 * Math.PI);

      //壁層を作成
      WallLayer[] wlEW = new WallLayer[]
      {
        new WallLayer("石膏ボード", 0.220, 830, 0.008),
        new WallLayer("コンクリート", 1.600, 2000, 0.150)
      };
      WallLayer[] wlIW = new WallLayer[]
      {
        new WallLayer("石膏ボード", 0.220, 830, 0.008),
        new AirGapLayer("非密閉中空層", false, 0.050),
        new WallLayer("石膏ボード", 0.220, 830, 0.008)
      };
      WallLayer[] wlFL = new WallLayer[]
      {
        new WallLayer("ロックウール化粧吸音板", 0.064, 290, 0.012),
        new WallLayer("石膏ボード", 0.220, 830, 0.008),
        new AirGapLayer("非密閉中空層", false, 0.050),
        new WallLayer("コンクリート", 1.600, 2000, 0.150),
        new AirGapLayer("非密閉中空層", false, 0.050),
        new WallLayer("ビニル系床材", 0.190, 2000, 0.003)
      };

      //壁を作成
      Wall[] walls = new Wall[4];
      walls[0] = new Wall(3.0 * 5.0, wlFL); //床天井
      walls[1] = new Wall(5.0 * 4.0, wlIW); //東西内壁
      walls[2] = new Wall(3.0 * 4.0, wlIW); //北内壁
      walls[3] = new Wall(3.0 * 4.0 - 2.8 * 1.8, wlEW); //南外壁

      //窓を作成
      Window[] windows = new Window[1];
      windows[0] = new Window(2.8 * 1.8,
        new double[] { 0.7 },  //透過率70%
        new double[] { 0.1 },  //反射率10%
        incS);

      //ゾーンを作成
      Zone[] zones = new Zone[1];
      zones[0] = new Zone("計算対象室", 3.0 * 5.0 * 4.0 * 1.2, 3.0 * 5.0);
      zones[0].HeatCapacity = zones[0].AirMass * 10;
      zones[0].VentilationRate = zones[0].AirMass * 0.3 / 3600d;

      //多数室を作成
      MultiRooms mRoom = new MultiRooms(1, zones, walls, windows);
      mRoom.AddZone(0, 0);
      mRoom.AddWindow(0, 0);
      mRoom.AddWall(0, 0, true); mRoom.AddWall(0, 0, false); //床天井・無限ループ
      mRoom.AddWall(0, 1, true); mRoom.AddWall(0, 1, false); //東西内壁・無限ループ
      mRoom.AddWall(0, 2, true); mRoom.UseAdjacentSpaceFactor(2, false, 0.4); //北内壁
      mRoom.AddWall(0, 3, true); mRoom.SetOutsideWall(3, false, incS); //南外壁

      BuildingThermalModel bModel = new BuildingThermalModel(new MultiRooms[] { mRoom });
      bModel.InitializeAirState(24, 0.028);
      bModel.TimeStep = 3600;

      return bModel;
    }

  }
}
